#!/bin/bash

{{/*
Copyright 2017 The Openstack-Helm Authors.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

   http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/}}

set -ex

function retry {
    local retries=$1
    shift
    local msg="$1"
    shift

    local count=0
    until "$@"; do
        exit=$?
        wait=$((2 * $count))
        count=$(($count + 1))
        if [ $count -lt $retries ]; then
            echo "Retry $count/$retries exited $exit, retrying in $wait seconds..."
            sleep $wait
        else
            echo "Retry $count/$retries exited $exit, no more retries left."
            echo "$msg"
            return $exit
        fi
    done
    return 0
}

function amphora_manage {
  local _image_filename=/tmp/amphora.qcow2

  # get new updated image
  curl "$OCTAVIA_AMPHORA_IMAGE_URL" --output $_image_filename --retry 5 --retry-delay 30 --fail
  ACTUAL_IMAGE_CHECKSUM=$(md5sum $_image_filename | cut -d' ' -f1)

  if [[ -n "$OCTAVIA_AMPHORA_IMAGE_CHECKSUM" && "$ACTUAL_IMAGE_CHECKSUM" != "$OCTAVIA_AMPHORA_IMAGE_CHECKSUM" ]]; then
    echo 'ERROR: Got broken/incorrect image' >&2
    exit 1
  fi

  CURRENT_IMAGE_INFO=($(openstack image show "$OCTAVIA_OSH_LB_AMPHORA_IMAGE_NAME" -cchecksum -cid -fvalue 2>/dev/null || true))

  if [[ "$ACTUAL_IMAGE_CHECKSUM" != "${CURRENT_IMAGE_INFO[0]}" ]]; then
    # upload image to glance
    openstack image create --disk-format qcow2 --container-format bare --public --file $_image_filename --tag amphora "$OCTAVIA_OSH_LB_AMPHORA_IMAGE_NAME"

    if (( ${#CURRENT_IMAGE_INFO[@]} )); then
      # remove outdated image
      openstack image delete "${CURRENT_IMAGE_INFO[1]}"
    fi
  fi

  rm -f $_image_filename
}

function get_or_create_lb-mgmt-sec-grp {
  function get_or_create_retry {
    openstack security group show lb-mgmt-sec-grp -f value -c id
  }

  retry 7 "Failed to get SecurityGroup lb-mgmt-sec-grp" get_or_create_retry
}

function get_or_create_lb-health-mgr-sec-grp {
  function get_or_create_retry {
    openstack security group show lb-health-mgr-sec-grp -f value -c id
  }

  retry 7 "Failed to get SecurityGroup lb-health-mgr-sec-grp" get_or_create_retry
}

echo $(hostname)
kubectl version

OCTAVIA_HOSTS=$(kubectl get node --selector="${OCTAVIA_NODE_SELECTOR}" --output=jsonpath={.items..metadata.name})

for HOST in ${OCTAVIA_HOSTS}; do
  function _retry_ovs_agents_up() {
     kubectl -n openstack get pods -l application=neutron,component=neutron-ovs-agent --field-selector=status.phase=Running -o wide |grep -q $HOST
  }

  retry 50 "Some ovs agents are still down." _retry_ovs_agents_up
done

amphora_manage

network_opts='lb-mgmt-net -fvalue -cmtu'
NETWORK_MTU=$(openstack network show $network_opts 2>/dev/null || openstack network create $network_opts)

declare -A SUBNETS_MASKS
while IFS= read -r subnet; do
  range=$(jq -r .range <<< $subnet)
  pool_start=$(jq -r .pool_start <<< $subnet)
  pool_end=$(jq -r .pool_end <<< $subnet)
  lb_mgmt_subnet="lb-mgmt-subnet-$(echo $range | tr -d './')"
  subnet_opts="$lb_mgmt_subnet -fvalue -cid"
  subnet_id=$(openstack subnet show $subnet_opts 2>/dev/null || openstack subnet create --subnet-range ${range} --allocation-pool start=${pool_start},end=${pool_end} --network lb-mgmt-net $subnet_opts)
  SUBNETS_MASKS[$subnet_id]=${range#*/}
  FIXED_IP_SUBNETS+="--fixed-ip subnet=$subnet_id "
done <<< $(echo ${OCTAVIA_OSH_LB_SUBNETS} | jq -r -c .[])

get_or_create_lb-mgmt-sec-grp || openstack security group create lb-mgmt-sec-grp
openstack security group rule create --protocol icmp lb-mgmt-sec-grp || true
openstack security group rule create --protocol tcp --dst-port 22 lb-mgmt-sec-grp || true
openstack security group rule create --protocol tcp --dst-port ${OCTAVIA_OSH_LB_HM_REMOTE_PORT} lb-mgmt-sec-grp || true

get_or_create_lb-health-mgr-sec-grp || openstack security group create lb-health-mgr-sec-grp
openstack security group rule create --protocol udp --dst-port ${OCTAVIA_OSH_LB_HM_HOST_PORT} lb-health-mgr-sec-grp || true

declare -A PORTS
CONTROLLER_IP_LIST=''
for HOST in ${OCTAVIA_HOSTS}; do
  port_opts="octavia-health-manager-listen-port-$HOST --noindent -fjson -cfixed_ips -cmac_address -cid"
  PORTS[$HOST]=$(openstack port show $port_opts 2>/dev/null || openstack port create $port_opts --security-group lb-health-mgr-sec-grp --device-owner Octavia:health-mgr --host=$HOST $FIXED_IP_SUBNETS --network lb-mgmt-net)
  CONTROLLER_IP_LIST=${CONTROLLER_IP_LIST}$(jq -r .fixed_ips[0].ip_address <<< ${PORTS[$HOST]}):${OCTAVIA_OSH_LB_HM_HOST_PORT},
done

#find service project ID according to domain
if [[ "${OS_PROJECT_DOMAIN_NAME}" == "Default" ]]
then
  PROJECT_DOMAIN_ID="default"
else
  PROJECT_DOMAIN_ID=$(openstack domain show -f value -c id \
    "${OS_PROJECT_DOMAIN_NAME}")
fi
USER_PROJECT_ID=$(openstack project show -f value -c id \
    --domain="${PROJECT_DOMAIN_ID}" \
    "${OS_PROJECT_NAME}")
#set unlimited quotas for octavia project
openstack quota set ${USER_PROJECT_ID} \
  --instances -1 \
  --key-pairs -1  \
  --secgroup-rules -1 \
  --secgroups -1 \
  --ports -1 \
  --fixed-ips -1 \
  --cores -1 \
  --ram -1

openstack flavor show m1.amphora -f value -c id || openstack flavor create --id auto --ram 1024 --disk 2 --vcpus 1 --private m1.amphora
openstack keypair show octavia_ssh_key -f value -c id || openstack keypair create --public-key /tmp/.ssh/octavia_ssh_key.pub octavia_ssh_key

OWNER_UID=$(kubectl get configmaps octavia-bin -o jsonpath='{.metadata.uid}' -n openstack)

function get_ports_info() {
  # OCTAVIA_HOSTS is a string with hosts where octavia resides,
  # e.g. 'host-1 host-2 ... host-N'
  for HOST in ${OCTAVIA_HOSTS}; do
    PORT_ADDRS=
    while IFS= read -r fixed_ip; do
      PORT_ADDRS+="$(jq -r .ip_address <<< $fixed_ip)/${SUBNETS_MASKS[$(jq -r .subnet_id <<< $fixed_ip)]} "
    done <<< $(echo ${PORTS[$HOST]} | jq -r -c .fixed_ips[])
    echo "    "PORT_MAC_${HOST//-/_}=$(jq -r .mac_address <<< ${PORTS[$HOST]})
    echo "    "PORT_ID_${HOST//-/_}=$(jq -r .id <<< ${PORTS[$HOST]})
    echo "    PORT_ADDRS_${HOST//-/_}='$PORT_ADDRS'"
  done
}

function create_configmap_settings() {
local amp_image_owner_id
local amp_secgroup_list
local amp_flavor_id
local amp_boot_network_list

amp_image_owner_id=$(openstack image show $OCTAVIA_OSH_LB_AMPHORA_IMAGE_NAME -f value -c owner)
amp_secgroup_list=$(openstack security group show lb-mgmt-sec-grp -f value -c id)
amp_flavor_id=$(openstack flavor show m1.amphora -f value -c id)
amp_boot_network_list=$(openstack network list --name lb-mgmt-net -f value -c ID)
  {
  cat <<EOF
---
apiVersion: v1
kind: ConfigMap
metadata:
  name: octavia-settings
  namespace: openstack
  ownerReferences:
  - apiVersion: v1
    name: octavia-bin
    kind: ConfigMap
    controller: true
    uid: $OWNER_UID
data:
  settings.conf: |
    [controller_worker]
    amp_image_owner_id=${amp_image_owner_id}
    amp_secgroup_list=${amp_secgroup_list}
    amp_flavor_id=${amp_flavor_id}
    amp_boot_network_list=${amp_boot_network_list}
    [health_manager]
    bind_port=${OCTAVIA_OSH_LB_HM_HOST_PORT}
    controller_ip_port_list=${CONTROLLER_IP_LIST}
  ports_configs: |
    NETWORK_MTU=${NETWORK_MTU}
$(get_ports_info)
EOF
  } | kubectl apply --namespace openstack -f -
}

create_configmap_settings
